"""
Handles deserialization / decoding of responses
"""

import gzip
import io
import json
import logging
import traceback
from collections import defaultdict
from email.message import EmailMessage
from typing import Dict, Tuple

from indico.errors import IndicoDecodingError

logger = logging.getLogger(__name__)


def decompress(response):
    response.raw.decode_content = True
    value = io.BytesIO(response.raw.data).getvalue()
    return gzip.decompress(value)


def deserialize(response, force_json=False, force_decompress=False):
    content_type, params = parse_header(response.headers.get("Content-Type"))

    if force_decompress or content_type in ["application/x-gzip", "application/gzip"]:
        content = decompress(response)
    else:
        content = response.content

    charset = params.get("charset", "utf-8")

    # For storage object for example where the content is json based on url ending
    if force_json:
        content_type = "application/json"

    try:
        return _SERIALIZATION_FNS[content_type](content, charset)
    except Exception:
        logger.debug(traceback.format_exc())
        raise IndicoDecodingError(
            content_type, charset, content.decode("ascii", "ignore")
        )


async def aio_deserialize(response, force_json=False, force_decompress=False):
    content_type, params = parse_header(response.headers.get("Content-Type"))
    content = await response.read()

    if force_decompress or content_type in ["application/x-gzip", "application/gzip"]:
        content = gzip.decompress(io.BytesIO(content).get_value())

    charset = params.get("charset", "utf-8")

    # For storage object for example where the content is json based on url ending
    if force_json:
        content_type = "application/json"

    try:
        return _SERIALIZATION_FNS[content_type](content, charset)
    except Exception:
        logger.debug(traceback.format_exc())
        raise IndicoDecodingError(
            content_type, charset, content.decode("ascii", "ignore")
        )


def parse_header(header: str) -> Tuple[str, Dict[str, str]]:
    email = EmailMessage()
    email["Content-Type"] = header
    return email.get_content_type(), email["Content-Type"].params


def raw_bytes(content, *args, **kwargs):
    return content


def msgpack_deserialization(content, charset):
    try:
        import msgpack
    except ImportError as error:
        raise RuntimeError(
            "msgpack deserialization requires additional dependencies: "
            "`pip install indico-client[deserialization]`"
        ) from error

    return msgpack.unpackb(content)


def json_deserialization(content, charset="utf-8"):
    return json.loads(content.decode(charset))


def text_deserialization(content, charset="utf-8"):
    return content.decode(charset)


def image_serialization(content, charset=None):
    return content


def zip_serialization(content, charset=None):
    return content


_SERIALIZATION_FNS = defaultdict(
    lambda: text_deserialization,
    {
        "application/pdf": raw_bytes,
        "application/octet-stream": raw_bytes,
        "application/doc": raw_bytes,
        "application/ms-doc": raw_bytes,
        "application/msword": raw_bytes,
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document": raw_bytes,
        "application/vnd.oasis.opendocument.text": raw_bytes,
        "application/zip": zip_serialization,
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": raw_bytes,
        "application/vnd.ms-excel": raw_bytes,
        "application/msexcel": raw_bytes,
        "application/excel": raw_bytes,
        "application/msexcel": raw_bytes,
        "application/x-dos_ms_excel": raw_bytes,
        "application/x-excel": raw_bytes,
        "application/x-ms-excel": raw_bytes,
        "application/x-xls": raw_bytes,
        "application/xlc": raw_bytes,
        "application/xls": raw_bytes,
        "application/xlt": raw_bytes,
        "application/vnd.ms-powerpoint": raw_bytes,
        "application/vnd.openxmlformats-officedocument.presentationml.presentation": raw_bytes,
        "application/mspowerpoint": raw_bytes,
        "application/powerpoint": raw_bytes,
        "application/vnd.ms-powerpoint": raw_bytes,
        "application/x-mspowerpoint": raw_bytes,
        "image/png": image_serialization,
        "image/jpeg": image_serialization,
        "image/jpg": image_serialization,
        "image/tiff": image_serialization,
        "text/html": text_deserialization,
        "text/plain": text_deserialization,
        "text/rtf": text_deserialization,
        "message/rfc822": raw_bytes,
        "application/vnd.ms-outlook": raw_bytes,
        "application/vnd.ms-office": raw_bytes,
        "binary/octet-stream": raw_bytes,
        "application/rtf": text_deserialization,
        "text/csv": text_deserialization,
        "application/x-msgpack": msgpack_deserialization,
        "application/msgpack": msgpack_deserialization,
        "x-msgpack": msgpack_deserialization,
        "msgpack": msgpack_deserialization,
        "application/json": json_deserialization,
        "json": json_deserialization,
        "application/javascript": json_deserialization,
        "application/x-gzip": raw_bytes,
        "application/gzip": raw_bytes,
    },
)



import json
from typing import Any, Dict, Optional

from indico.types.base import BaseType, JSONType


class TrainingProgress(BaseType):
    """
    Model training progress

    Shows percent complete for a model training

    Attributes:
        percent_complete (float): Percent complete for a model that's training
    """

    percent_complete: float


class Model(BaseType):
    """
    A Model in the Indico Platform.

    Models are a part of a Model Group

    Attributes:
        id (int): The model id. This is different from the model group id.
        status (str): "CREATED", "TRAINING", "COMPLETE" or "FAILED"
        training_progress (TrainingProgress): Percent complete training progress on a model.
    """

    id: int
    status: str
    training_progress: TrainingProgress


class ModelOptions(BaseType):
    """
    Model Options in the Indico Platform.

    Attributes:
        id (int): The model options id
        domain (str): feature domain
        high_quality (bool): flag denoting if high quality was specified
        interlabeler_resolution (str): label resolution strategy specified
        sampling_strategy (str): sampling strategy specified
        seed (int): number of seed specified
        test_split (float): denotes specified test split
        weight_by_class_frequency (bool): flag denoting weight by class frequency
        word_predictor_strength (str): word predictor strength specified
        predict_options (dict): JSONString representation of the predict options specified
        model_training_options (dict): JSONString representation of the model training options specified
    """

    id: int
    domain: str
    high_quality: bool
    interlabeler_resolution: str
    sampling_strategy: str
    seed: int
    test_split: float
    weight_by_class_frequency: bool
    word_predictor_strength: str
    predict_options: JSONType
    model_training_options: JSONType



from datetime import datetime
from enum import Enum
from indico.types import BaseType


class IntegrationType(Enum):
    EXCHANGE = 1

class ExchangeIntegrationCredentials(BaseType):
    """
    Credentials needed to connect a Microsoft Exchange server to an Indico workflow using
    """
    client_id: str
    client_secret: str
    tenant_id: str

class ExchangeIntegrationConfiguration(BaseType):
    """
    Configuration options available for an integration with Microsoft Exchange
    """
    user_id: str
    folder_id: str
    folder_name: str = None
    filters: str = None


class Integration(BaseType):
    """
    An integration pulls document from a third-party data source and submits them to a workflow

    Args:
        id(int): ID of the integration 
        workflow_id(int): ID of the workflow to submit to.
        enabled(bool): Whether Indico is currently sending documents from this datasource to the workflow.
        created_at(datetime): When this integration was created.
        integration_type(IntegrationType): Type of the data source for this integration
    """

    id: int
    workflow_id: int
    enabled: bool
    created_at: datetime
    integration_type: IntegrationType

class ExchangeIntegration(Integration):
    config: ExchangeIntegrationConfiguration



from .datafile import *
from .dataset import *
from .jobs import *
from .model_group import *
from .model import *
from .output_file import *
from .submission_file import *
from .submission import *
from .workflow import *
from .questionnaire import Example
from .integration import *
from .model_export import *



from typing import List
from enum import Enum

from indico.types.base import BaseType


class LabelResolutionStrategy(Enum):
    MAJORITY_VOTE_WITH_TIES = "majority_vote_with_ties"
    MAJORITY_VOTE_WITHOUT_TIES = "majority_vote_without_ties"
    UNANIMOUS = "unanimous"
    ALL = "all"


class Export(BaseType):
    """
    An export of a dataset.

    Attributes:
        id (int): Id of the export
        dataset_id (int): Dataset id of the export
        name (str): Name of the export
        status (ExportStatus): Export job status
        column_ids (List(int)): Data columns for export
        labelset_id (int): Labelset column for export
        model_ids (List(int)): Models to include predictions from
        frozen_labelset_ids (List(int)): Frozen labelsets to select examples
        num_labels (int): Number of labels on the dataset
        anonymous (bool): Whether to anonymize labelers
        download_url (str): Indico url of the export csv
        created_at (str): Unix timestamp for when the export was created
    """

    id: int
    dataset_id: int
    name: str
    status: str
    column_ids: List[int]
    labelset_id: int
    model_ids: List[int]
    frozen_labelset_ids: List[int]
    num_labels: int
    anonymous: bool
    download_url: str
    created_at: str



import pytest
from datetime import datetime

from indico.client import IndicoClient, IndicoConfig
from indico.filters import DateRangeFilter, SubmissionFilter, SubmissionReviewFilter
from indico.queries import ListSubmissions, GetReviews
from indico.types import SubmissionReviewFull


def test_list_submissions(indico):
    client = IndicoClient()

    subs = client.call(ListSubmissions(limit=10))
    assert len(subs) == 10


def test_list_submissions_filter_filetype(indico):
    client = IndicoClient()

    subs = client.call(ListSubmissions(filters=SubmissionFilter(file_type=["PDF"]), limit=10))
    assert len(subs) > 0
    for sub in subs:
        sub_filetype = sub.input_filename.split(".")[-1].upper()
        assert sub_filetype == "PDF" or sub_filetype.lower() == sub.input_filename


@pytest.mark.parametrize(
    "_input_filename, _should_contain",
    [
        ("pdf", True),
        ("randomstring", False)
    ],
)
def test_list_submissions_filter_filename(indico, _input_filename, _should_contain):
    client = IndicoClient()

    subs = client.call(ListSubmissions(filters=SubmissionFilter(input_filename=_input_filename), limit=10))
    for sub in subs:
        assert (_input_filename in sub.input_filename) == _should_contain


def test_list_submissions_filter_reviews(indico):
    client = IndicoClient()

    review_filter = SubmissionReviewFilter(rejected=False)

    subs = client.call(ListSubmissions(filters=SubmissionFilter(reviews=review_filter), limit=10))
    assert len(subs) >= 0

def test_list_submissions_filter_reviews_in_progress(indico):
    client = IndicoClient()

    subs = client.call(ListSubmissions(filters=SubmissionFilter(review_in_progress=False), limit=10))
    assert len(subs) > 0


def test_list_submissions_filter_created_at(indico):
    client = IndicoClient()

    date_filter = DateRangeFilter(
        filter_from=datetime(year=2020, month=2, day=2).strftime("%Y-%m-%d"),
        filter_to=datetime.now().strftime("%Y-%m-%d")
    )
    subs = client.call(ListSubmissions(filters=SubmissionFilter(created_at=date_filter), limit=10))
    assert len(subs) > 0
    subs = client.call(ListSubmissions(filters=SubmissionFilter(updated_at=date_filter), limit=10))
    assert len(subs) > 0


def test_get_reviews(indico):
    client = IndicoClient()

    subs = client.call(ListSubmissions(limit=10))
    assert len(subs) > 0
    for sub in subs:
        reviews = client.call(GetReviews(sub.id))
        for review in reviews:
            assert (isinstance(review, SubmissionReviewFull))
            if not review.rejected:
                assert isinstance(review.changes, (list, dict))



import datetime

from indico.types.base import BaseType
from typing import List


class AppRoles(BaseType):
    """Info about roles. Name and how many users have this role.
    
    Attributes:
        role (str): A role name
        count (int): Count of how many users have said role.
    """
    role: str
    count: int


class UserDatasets(BaseType):
    """Dataset Id and roles assigned to user in that dataset.
    
    Attributes:
        dataset_id (int): The id of a particular dataset
        role (str): A dataset assigned to said dataset.
    """
    dataset_id: int
    role: str


class UserSummaryCounts(BaseType):
    """
    Summary of user counts

    Attributes:
        enabled (int): How many user accounts are enabled.
        disabled (int): How many user accounts are disabled.
    """
    enabled: int
    disabled: int


class UserSnapshot(BaseType):
    """Individual information about a user and their dataset access
    
    Attributes:
        id (int): The user's id.
        name (str): The user's name.
        email (str): The user's email.
        created_at (str): Time of creation.
        enabled (bool): True if user account is enabled.
        roles (List[str]): List of roles assigned to this user.
        datasets (List[UserDatasets]): List of datasets this user can access.
    """
    id: int
    name: str
    email: str
    created_at: str
    enabled: bool
    roles: List[str]
    datasets: List[UserDatasets]


class UserSummary(BaseType):
    """Summary data on users and app roles
    
    Attributes:
        users (UserSummarycounts): Counts of enabled/disabled users.
        app_roles (List[AppRoles]): List of all available user roles.
    """
    users: UserSummaryCounts
    app_roles: List[AppRoles]


class DatasetRole(BaseType):
    """Dataset role information
    
    Attributes:
        dataset_id (int): Id of a particular dataset.
        role (str): Role name which has access to this dataset.
    """
    dataset_id: int
    role: str


class UserChangelog(BaseType):
    """Log entry of a change made to a user's permission
    
    Attributes:
        id (str): Id of the log entry
        date (datetime): Time of long entry.
        user_id (int): Id of the user whose attributes were changed.
        user_email (str): Email of the user whose attributes were changed.
        updated_by (id): Id of the account which made the change.
        updater_email (str): Email of the account which made the change.
        previously_enabled (bool): True if this account was enabled prior to change.
        enabled (bool): True if account is enabled after the change.
        previous_roles (List[str]): Roles assigned prior to the change.
        roles (List[str]): Roles assigned after the change.
        previous_datasets (List[DatasetRole]): List of dataset/role mappings prior to the change.
        datasets (List[DatasetRole]): List of dataset/role mappings after the change.
        changes_made (List[str]): A list of changed made.

    """
    id: str
    date: datetime
    user_id: int
    user_email: str
    updated_by: int
    updater_email: str
    previously_enabled: bool
    enabled: bool
    previous_roles: List[str]
    roles: List[str]
    previous_datasets: List[DatasetRole]
    datasets: List[DatasetRole]
    changes_made: List[str]


class UserChangelogReport(BaseType):
    """Job id of a request for a changelog report file for download
    
    Attributes:
        job_id (str): The job id. Use for fetching JobStatus.
    """
    job_id: str



from typing import List, Optional

from indico.client.request import Delay, GraphQLRequest, RequestChain
from indico.errors import IndicoError, IndicoNotFound
from indico.types.questionnaire import Example, Questionnaire


class AddLabels(GraphQLRequest):
    """
    Add labels to an existing labelset.

    Args:
        labelset_id (int): The id of the labelset to add labels to.
        labels (List(dict)): A list of dicts containing rowIndex and target fields for the data points to add.
        model_group_id (int, optional): The id of the model group to add labels to.

    Raises:
        IndicoInputError if the length of targets and row_index are not equal

    """

    query = """
        mutation(
            $labels: [LabelInput]!,
            $labelset_id: Int!,
        ){
            submitLabelsV2(
                labels: $labels
                labelsetId: $labelset_id,
            ){ success }
        }
        """

    def __init__(
        self,
        labelset_id: int,
        labels: List[dict],
        model_group_id: int = None,
    ):
        super().__init__(
            query=self.query,
            variables={
                "labels": labels,
                "labelset_id": labelset_id,
                "model_group_id": None,
            },
        )


class GetQuestionnaireExamples(GraphQLRequest):
    """
    Gets unlabeled examples from a Questionnaire.

    Args:
        questionnaire_id (int): The id of the questionnaire to get examples from.
        num_examples (int): The number of examples to get from the questionnaire.

    Returns:
        List[Example]

    """

    query = """
    query(
        $questionnaire_id: Int!,
        $num_examples: Int!,
        $datafile_id: Int
    )
    {
        questionnaires(questionnaireIds: [$questionnaire_id]) {
            questionnaires {
                examples(numExamples: $num_examples, datafileId: $datafile_id) {
                    rowIndex
                    datafileId
                    source
                    id
                }
            }
        }
    }
    """

    def __init__(
        self,
        questionnaire_id: int,
        num_examples: int,
        datafile_id: Optional[int] = None,
    ):
        super().__init__(
            query=self.query,
            variables={
                "questionnaire_id": questionnaire_id,
                "num_examples": num_examples,
                "datafile_id": datafile_id,
            },
        )

    def process_response(self, response):
        try:
            examples = [
                Example(**e)
                for e in super().process_response(response)["questionnaires"][
                    "questionnaires"
                ][0]["examples"]
            ]
        except IndexError:
            raise IndicoNotFound(
                "Examples not found. Please check the ID you are using."
            )
        return examples


class _GetQuestionnaire(GraphQLRequest):
    """
    Gets a questionnaire from an ID.

    Args:
        questionaire_id (int): The id of the questionnaire to get examples from.

    Returns
        Questionnaire
    """

    query = """
    query(
        $questionnaire_id: Int!
    )
    {
        questionnaires(questionnaireIds: [$questionnaire_id]) {
            questionnaires {
                id
                questionsStatus
                odl
                name
                numTotalExamples
                numFullyLabeled
                question {
                    labelset{
                        targetNames{
                            id
                            name
                        }
                    }
                }
            }
        }
    }
    """

    def __init__(self, questionnaire_id: int):
        super().__init__(
            query=self.query,
            variables={"questionnaire_id": questionnaire_id},
        )

    def process_response(self, response):
        questionnaire_list = super().process_response(response)["questionnaires"][
            "questionnaires"
        ]
        if not questionnaire_list or not questionnaire_list[0]:
            raise IndicoError("Cannot find questionnaire")
        return Questionnaire(**questionnaire_list[0])


class GetQuestionnaire(RequestChain):
    """
    Gets a questionnaire from an ID.

    Args:
        questionaire_id (int): The id of the questionnaire to get examples from.
        wait (bool, optional): Wait for the questionnaire to reach a COMPLETE status. Defaults to True.

    Returns:
        Questionnaire object
    """

    previous = None

    def __init__(self, questionnaire_id: int, wait: bool = True):
        self.questionnaire_id = questionnaire_id
        self.wait = wait

    def requests(self):
        yield _GetQuestionnaire(questionnaire_id=self.questionnaire_id)
        if self.wait:
            while self.previous.questions_status == "STARTED":
                yield _GetQuestionnaire(questionnaire_id=self.questionnaire_id)
                yield Delay()



import time

import pytest

from indico.client import GraphQLRequest, IndicoClient
from indico.queries import (
    CreateWorkflow,
    DeleteWorkflowComponent,
    GetDataset,
    GetWorkflow,
    JobStatus,
)
from indico.queries import (
    AddLinkClassificationComponent,
    AddModelGroupComponent,
    AddStaticModelComponent,
    ProcessStaticModelExport,
    UploadStaticModelExport,
)
from indico.types import (
    Job,
    ModelGroup,
    ModelTaskType,
    NewLabelsetArguments,
    StaticModelConfig,
)

from ..data.datasets import *  # noqa


def test_add_mg_new_labelset(indico, org_annotate_dataset):
    client = IndicoClient()
    workflowreq = CreateWorkflow(
        dataset_id=org_annotate_dataset.id, name=f"OrgAnnotate-test-{int(time.time())}"
    )
    wf = client.call(workflowreq)
    mg_name = f"TestAnnotationModelGroup-{int(time.time())}"
    labelset_name = "test-create-labelset"
    after_component_id = wf.component_by_type("INPUT_OCR_EXTRACTION").id
    source_column_id = org_annotate_dataset.datacolumn_by_name(
        "News Headlines w/Company Names"
    ).id
    modelgroupreq = AddModelGroupComponent(
        name=mg_name,
        dataset_id=org_annotate_dataset.id,
        workflow_id=wf.id,
        after_component_id=after_component_id,
        source_column_id=source_column_id,
        new_labelset_args=NewLabelsetArguments(
            name=labelset_name,
            task_type=ModelTaskType.ANNOTATION,
            target_names=["target 1", "target 2"],
            datacolumn_id=source_column_id,
        ),
    )

    wf = client.call(modelgroupreq)
    dataset = client.call(GetDataset(id=org_annotate_dataset.id))
    assert dataset.labelset_by_name(labelset_name)


def test_delete_workflow_component(
    indico, airlines_dataset, airlines_workflow, airlines_model_group: ModelGroup
):
    client = IndicoClient()
    # get workflow that includes mg component
    wf = client.call(GetWorkflow(workflow_id=airlines_workflow.id))
    num_components = len(wf.components)
    num_links = len(wf.component_links)
    mg_comp_id = next(c.id for c in wf.components if c.component_type == "MODEL_GROUP")
    wf = client.call(
        DeleteWorkflowComponent(
            workflow_id=airlines_workflow.id, component_id=mg_comp_id
        )
    )
    assert len(wf.components) == num_components - 1
    assert mg_comp_id not in {c.id for c in wf.components}
    assert len(wf.component_links) == num_links - 1


def test_add_many_filtered_classes(indico, org_annotate_dataset):
    client = IndicoClient()
    workflowreq = CreateWorkflow(
        dataset_id=org_annotate_dataset.id, name=f"OrgAnnotate-test-{int(time.time())}"
    )
    wf = client.call(workflowreq)
    mg_name = f"TestAnnotationModelGroup-{int(time.time())}"
    labelset_name = "test-filtered-classes"
    after_component_id = next(
        c.id for c in wf.components if c.component_type == "INPUT_OCR_EXTRACTION"
    )
    source_column_id = org_annotate_dataset.datacolumn_by_name(
        "News Headlines w/Company Names"
    ).id
    modelgroupreq = AddModelGroupComponent(
        name=mg_name,
        dataset_id=org_annotate_dataset.id,
        workflow_id=wf.id,
        after_component_id=after_component_id,
        source_column_id=source_column_id,
        new_labelset_args=NewLabelsetArguments(
            name=labelset_name,
            task_type=ModelTaskType.CLASSIFICATION_MULTIPLE,
            target_names=["type 1", "type 2", "type 3"],
            datacolumn_id=source_column_id,
        ),
    )
    wf = client.call(modelgroupreq)
    mg = next(c for c in wf.components if c.component_type == "MODEL_GROUP")
    after_component_id = mg.id

    classes_to_filter = [["type 1"], ["type 3"]]
    filtered = AddLinkClassificationComponent(
        workflow_id=wf.id,
        after_component_id=after_component_id,
        model_group_id=mg.model_group.id,
        filtered_classes=classes_to_filter,
        labels="actual",
    )
    wf = client.call(filtered)
    new_component = next(
        c.id for c in wf.components if c.component_type == "LINK_CLASSIFICATION_MODEL"
    )
    assert new_component is not None


def test_add_single_filtered_class(indico, org_annotate_dataset):
    client = IndicoClient()
    workflowreq = CreateWorkflow(
        dataset_id=org_annotate_dataset.id, name=f"OrgAnnotate-test-{int(time.time())}"
    )
    wf = client.call(workflowreq)
    mg_name = f"TestAnnotationModelGroup-{int(time.time())}"
    labelset_name = "test-filtered-classes"
    after_component_id = next(
        c.id for c in wf.components if c.component_type == "INPUT_OCR_EXTRACTION"
    )
    source_column_id = org_annotate_dataset.datacolumn_by_name(
        "News Headlines w/Company Names"
    ).id
    modelgroupreq = AddModelGroupComponent(
        name=mg_name,
        dataset_id=org_annotate_dataset.id,
        workflow_id=wf.id,
        after_component_id=after_component_id,
        source_column_id=source_column_id,
        new_labelset_args=NewLabelsetArguments(
            name=labelset_name,
            task_type=ModelTaskType.CLASSIFICATION_MULTIPLE,
            target_names=["type 1", "type 2", "type 3"],
            datacolumn_id=source_column_id,
        ),
    )
    wf = client.call(modelgroupreq)
    mg = next(c for c in wf.components if c.component_type == "MODEL_GROUP")
    after_component_id = mg.id

    classes_to_filter = [["type 1"]]
    filtered = AddLinkClassificationComponent(
        workflow_id=wf.id,
        after_component_id=after_component_id,
        model_group_id=mg.model_group.id,
        filtered_classes=classes_to_filter,
        labels="actual",
    )
    wf = client.call(filtered)
    new_component = next(
        c.id for c in wf.components if c.component_type == "LINK_CLASSIFICATION_MODEL"
    )
    assert new_component is not None


def test_add_bad_syntax_filtered_classes(indico, org_annotate_dataset):
    client = IndicoClient()
    workflowreq = CreateWorkflow(
        dataset_id=org_annotate_dataset.id, name=f"OrgAnnotate-test-{int(time.time())}"
    )
    wf = client.call(workflowreq)
    mg_name = f"TestAnnotationModelGroup-{int(time.time())}"
    labelset_name = "test-filtered-classes"
    after_component_id = next(
        c.id for c in wf.components if c.component_type == "INPUT_OCR_EXTRACTION"
    )
    source_column_id = org_annotate_dataset.datacolumn_by_name(
        "News Headlines w/Company Names"
    ).id
    modelgroupreq = AddModelGroupComponent(
        name=mg_name,
        dataset_id=org_annotate_dataset.id,
        workflow_id=wf.id,
        after_component_id=after_component_id,
        source_column_id=source_column_id,
        new_labelset_args=NewLabelsetArguments(
            name=labelset_name,
            task_type=ModelTaskType.CLASSIFICATION_MULTIPLE,
            target_names=["type 1", "type 2", "type 3"],
            datacolumn_id=source_column_id,
        ),
    )
    wf = client.call(modelgroupreq)
    mg = next(c for c in wf.components if c.component_type == "MODEL_GROUP")
    after_component_id = mg.id

    classes_to_filter = [["type 1"], ["type 2"]]
    filtered = AddLinkClassificationComponent(
        workflow_id=wf.id,
        after_component_id=after_component_id,
        model_group_id=6108,
        filtered_classes=[classes_to_filter],
        labels="actual",
    )
    with pytest.raises(Exception):
        wf = client.call(filtered)


def test_upload_static_model_export(indico):
    client = IndicoClient()
    upload_req = UploadStaticModelExport(
        file_path="tests/integration/data/exports/test-export.zip"
    )
    storage_uri = client.call(upload_req)
    assert storage_uri is not None


def test_process_static_model_export(indico):
    client = IndicoClient()
    storage_uri = client.call(
        UploadStaticModelExport(
            file_path="tests/integration/data/exports/test-export.zip"
        )
    )
    job: Job = client.call(ProcessStaticModelExport(storage_uri=storage_uri))
    finished_job = client.call(JobStatus(job.id))
    assert finished_job.status == "SUCCESS"
    assert finished_job.result is not None


def test_add_static_model_component(indico, org_annotate_dataset):
    client = IndicoClient()
    workflowreq = CreateWorkflow(
        dataset_id=org_annotate_dataset.id, name=f"OrgAnnotate-test-{int(time.time())}"
    )
    wf = client.call(workflowreq)

    storage_uri = client.call(
        UploadStaticModelExport(
            file_path="tests/integration/data/exports/test-export.zip"
        )
    )
    job: Job = client.call(ProcessStaticModelExport(storage_uri=storage_uri))
    finished_job = client.call(JobStatus(job.id))
    assert finished_job.status == "SUCCESS"
    assert finished_job.result is not None

    after_component_id = next(
        c.id for c in wf.components if c.component_type == "INPUT_OCR_EXTRACTION"
    )

    static_model_req = AddStaticModelComponent(
        workflow_id=wf.id,
        after_component_id=after_component_id,
        static_component_config=StaticModelConfig(
            export_meta=finished_job.result,
        ),
    )
    wf = client.call(static_model_req)

    assert len(wf.components) == 3
    assert any(c.component_type == "STATIC_MODEL" for c in wf.components)



# -*- coding: utf-8 -*-

import asyncio
import time
from typing import Optional, Union

import urllib3

from indico.client.request import (
    GraphQLRequest,
    HTTPRequest,
    PagedRequest,
    RequestChain,
)
from indico.config import IndicoConfig
from indico.errors import IndicoError
from indico.client.request import Delay
from indico.http.client import AIOHTTPClient, HTTPClient


class IndicoClient:
    """
    The Indico GraphQL Client.

    IndicoClient is the primary way to interact with the Indico Platform.

    Args:
        config= (IndicoConfig, optional): IndicoConfig object with environment configuration

    Returns:
        IndicoConfig object

    Raises:
        RuntimeError: If api_token_path does not exist.
    """

    def __init__(self, config: Optional[IndicoConfig] = None):
        if not config:
            config = IndicoConfig()
        if not config.verify_ssl:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self.config = config
        self._http = HTTPClient(config)

    def _handle_request_chain(self, chain: RequestChain):
        response = None
        for request in chain.requests():
            if isinstance(request, HTTPRequest):
                response = self._http.execute_request(request)
                chain.previous = response
            elif isinstance(request, RequestChain):
                response = self._handle_request_chain(request)
                chain.previous = response
            elif isinstance(request, Delay):
                time.sleep(request.seconds)
        if chain.result:
            return chain.result
        return response

    def get_ipa_version(self):
        return self._http.execute_request(
            GraphQLRequest("query getIPAVersion {\n  ipaVersion\n}\n")
        )["ipaVersion"]

    def call(self, request: Union[HTTPRequest, RequestChain]):
        """
        Make a call to the Indico IPA Platform

        Args:
            request (GraphQLRequest or RequestChain): GraphQL request to send to the Indico Platform

        Returns:
            Response appropriate to the class of the provided request parameter. Often JSON, but not always.

        Raises:
            IndicoRequestError: With errors in processing the request
        """

        if isinstance(request, RequestChain):
            return self._handle_request_chain(request)
        elif request and isinstance(request, HTTPRequest):
            return self._http.execute_request(request)

    def paginate(self, request: PagedRequest):
        """
        Provides a generator that continues paging through responses
        Available with List<> Requests that offer pagination

        Example:
            for s in client.paginate(ListSubmissions()):
                print("Submission", s)
        """
        while request.has_next_page:
            r = self._http.execute_request(request)
            yield r


class AsyncIndicoClient:
    """
    The Async Indico GraphQL Client.
    Notably, this client does not offer the usage of
    `_disable_cookie_domain`

    You must explicitly create and close this client
        client = await AsyncIndicoClient(config=config).create()
        # ... do stuff
        await client.cleanup()

    or implicitly using a `with` statement
        async with AsyncIndicoClient(config=config) as client:
            # ... do stuff

    Args:
        config= (IndicoConfig, optional): IndicoConfig object with environment configuration

    Returns:
        IndicoConfig object

    Raises:
        RuntimeError: If api_token_path does not exist.
    """

    def __init__(self, config: Optional[IndicoConfig] = None):
        if not config:
            config = IndicoConfig()
        if not config.verify_ssl:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self.config = config
        self._http = AIOHTTPClient(config)
        self._created = False

    async def __aenter__(self):
        return await self.create()

    async def __aexit__(self, exc_type, exc, tb):
        await self.cleanup()

    async def create(self):
        await self._http.get_short_lived_access_token()
        self._created = True
        return self

    async def cleanup(self):
        await self._http.request_session.close()

    async def _handle_request_chain(self, chain: RequestChain):
        response = None
        for request in chain.requests():
            if isinstance(request, HTTPRequest):
                response = await self._http.execute_request(request)
                chain.previous = response
            elif isinstance(request, RequestChain):
                response = await self._handle_request_chain(request)
                chain.previous = response
            elif isinstance(request, Delay):
                await asyncio.sleep(request.seconds)
        if chain.result:
            return chain.result
        return response

    async def get_ipa_version(self):
        if not self._created:
            raise IndicoError("Please .create() your client")
        return (
            await self._http.execute_request(
                GraphQLRequest("query getIPAVersion {ipaVersion}")
            )
        )["ipaVersion"]

    async def call(self, request: Union[HTTPRequest, RequestChain]):
        """
        Make a call to the Indico IPA Platform

        Args:
            request (GraphQLRequest or RequestChain): GraphQL request to send to the Indico Platform

        Returns:
            Response appropriate to the class of the provided request parameter. Often JSON but not always.

        Raises:
            IndicoRequestError: With errors in processing the request
        """
        if not self._created:
            raise IndicoError("Please .create() your client")

        if isinstance(request, RequestChain):
            return await self._handle_request_chain(request)
        elif request and isinstance(request, HTTPRequest):
            return await self._http.execute_request(request)

    async def paginate(self, request: PagedRequest):
        """
        Provides a generator that continues paging through responses
        Available with List<> Requests that offer pagination

        Example:
            async for s in client.paginate(ListSubmissions()):
                print("Submission", s)
        """
        if not self._created:
            raise IndicoError("Please .create() your client")
        while request.has_next_page:
            r = await self._http.execute_request(request)
            yield r



import datetime
from typing import Optional

from indico.types import BaseType, List, JSONType

from . import OutputFile, SubmissionFile

VALID_SUBMISSION_STATUSES = [
    "COMPLETE",
    "FAILED",
    "PENDING_REVIEW",
    "PROCESSING",
    "PENDING_ADMIN_REVIEW",
    "PENDING_AUTO_REVIEW",
]

VALID_REVIEW_TYPES = [
    "MANUAL",
    "AUTO",
    "ADMIN",
]

SUBMISSION_RESULT_VERSIONS = ["ONE", "TWO", "THREE", "OLDEST_SUPPORTED", "LATEST"]


class SubmissionRetries(BaseType):
    """
    Information about a retried submission.

    Attributes:
        id (int): The id of the retry attempt.
        previous_errors (str): Errors from the previous attempt.
        previous_status (str): The status from the previous attempt.
        retry_errors (str): Errors encountered on this retry.
        submission_id (int): The ID of the submission being retried.
    """

    id: int
    previous_errors: str
    previous_status: str
    retry_errors: str
    submission_id: int


class SubmissionReview(BaseType):
    f"""
    Information about a submission's Reviews.

    Attributes:
        id (int): The ID of the review.
        submission_id (int): The ID of the submission that is being reviewed.
        created_at (str): Timestamp of when the document was checked out
        created_by (int): The ID of the User who submitted the review.
        completed_at (str): Timestamp of when the review was submitted.
        rejected (bool): Whether a submission has been rejected.
        review_type (str): Type of review. One of {VALID_REVIEW_TYPES}
        notes (str): Rejection reasons provided by user.
    """
    id: int
    submission_id: int
    created_at: str
    created_by: int
    completed_at: str
    rejected: bool
    review_type: str
    notes: str

class SubmissionReviewFull(BaseType):
    f"""
    Information about a submission's Reviews. Includes changes

    Attributes:
        id (int): The ID of the review.
        submission_id (int): The ID of the submission that is being reviewed.
        created_at (str): Timestamp of when the document was checked out
        created_by (int): The ID of the User who submitted the review.
        completed_at (str): Timestamp of when the review was submitted.
        rejected (bool): Whether a submission has been rejected.
        review_type (str): Type of review. One of {VALID_REVIEW_TYPES}
        notes (str): Rejection reasons provided by user.
        changes (dict): Changes for this review.
    """
    id: int
    submission_id: int
    created_at: str
    created_by: int
    completed_at: str
    rejected: bool
    review_type: str
    notes: str
    changes: JSONType


class Submission(BaseType):
    f"""
    A Submission in the Indico Platform.

    Submissions represent a single input which has been sent for processing by a
    specific workflow. A submission consists of SubmissionFiles, generally a PDF.
    Processing broadly consists of an input processor, a series of processors and
    components associated with particular docbots, and finally an output processor
    to generate the result file.

    Attributes:
        id (int): The Submission id
        dataset_id (int): The Dataset id
        workflow_id (int): The Workflow id
        status (str): status of the submission. One of
            {VALID_SUBMISSION_STATUSES}
        created_at (datetime): Datetime the submission was created
        updated_at (datetime): Datetime the submission was updated
        created_by (int): Id of the user who created the submission
        updated_by (int): Id of the user who updated the submission
        completed_at (datetime): Datetime the submission reached a completed state
        files_deleted (bool): Submission files have been deleted (True) or not deleted (False) from file store
        input_files (List[SubmissionFile]): The SubmissionFiles for the Submission
        input_file (str): URL of the first input datafile within the Indico Platform.
        input_filename (str): Name of the first original file
        result_file (str): URL of the latest result file for this submission
        output_files (List[OutputFile]): List of output files from submission
        retrieved (bool): Whether the submission has been retrieved by a user
            This flag is set manually by users.
        auto_review (SubmissionReview): Latest auto review for submission
        errors (str): Any errors raised while processing the submission
        retries (List[SubmissionRetries]): If requested, information about previous retries of this submission.
        reviews (List[SubmissionReview]): Completed reviews of this submission, without changes
        review_in_progress (bool): True if the submission is being actively reviewed
    """

    id: int
    dataset_id: int
    workflow_id: int
    status: str
    created_at: datetime
    updated_at: datetime
    created_by: int
    updated_by: int
    completed_at: datetime
    files_deleted: bool
    input_files: List[SubmissionFile]
    input_file: str
    input_filename: str
    result_file: str
    output_files: List[OutputFile]
    retrieved: bool
    auto_review: SubmissionReview
    errors: str
    retries: List[SubmissionRetries]
    reviews: List[SubmissionReview]
    review_in_progress: bool



# -*- coding: utf-8 -*-

import os

from pathlib import Path
from indico.errors import IndicoInvalidConfigSetting


class IndicoConfig:
    """
    Configuration for the IndicoClient.

    Args:
        host= (str, optional): Indico Platform hostname (eg mycluster.indico.io )
        api_token_path= (str, optional): Path to the Indico API token file indico_api_token.txt. Defaults to user's home directory. Ignored if api_token is provided.
        api_token= (str, optional): The actual text of the API Token. Takes precedence over api_token_path
        verify_ssl= (bool, optional): Whether to verify the host's SSL certificate. Default=True
        requests_params= (dict, optional): Dictionary of requests. Session parameters to set

    Returns:
        IndicoConfig object

    Raises:
        RuntimeError: If api_token_path does not exist.
    """

    host: str
    protocol: str
    serializer: str
    api_token_path: str
    api_token: str = None
    verify_ssl: bool = True
    requests_params: dict = None
    _disable_cookie_domain: bool = False

    def __init__(self, **kwargs):

        self.host: str = os.getenv("INDICO_HOST")
        self.protocol: str = os.getenv("INDICO_PROTOCOL", "https")
        self.serializer: str = os.getenv("INDICO_SERIALIZER", "msgpack")
        self.api_token_path: str = os.getenv("INDICO_API_TOKEN_PATH", Path.home())
        self.api_token: str = os.getenv("INDICO_API_TOKEN")

        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)
            else:
                raise IndicoInvalidConfigSetting(key)

        if not self.api_token:
            self.api_token_path, self.api_token = self._resolve_api_token()

    def _resolve_api_token(self):
        path = self.api_token_path
        if not isinstance(path, Path):
            path = Path(path)
        if not path.exists():
            path = Path.home()
        if not path.is_file():
            path = path / "indico_api_token.txt"

        if not path.exists():
            raise RuntimeError(
                "Expected indico_api_token.txt in home directory, "
                "or provided as indicoio.config.api_token_path"
            )

        with path.open("r") as f:
            return path, f.read().strip()



from typing import List
from indico.types.base import BaseType


class TargetName(BaseType):
    id: int
    name: str


class LabelSet(BaseType):
    target_names: List[TargetName]


class Question(BaseType):
    labelset: LabelSet


class Questionnaire(BaseType):
    id: int
    questions_status: str
    odl: bool
    name: str
    num_total_examples: int
    num_fully_labeled: int
    question: Question


class Example(BaseType):
    row_index: int
    datafile_id: int
    status: str
    source: str
    id: int


class QuestionnaireExamples(BaseType):
    examples: List[Example]



import pytest

from indico.client import IndicoClient, IndicoConfig
from indico.client.request import HTTPRequest, HTTPMethod, GraphQLRequest
from indico.errors import IndicoAuthenticationFailed
from tests.integration.data.datasets import org_annotate_dataset


def test_http_request(indico):
    client = IndicoClient()
    response = client.call(
        HTTPRequest(method=HTTPMethod.GET, path="/auth/users/details")
    )
    assert "email" in response


def test_bad_token(indico):
    with pytest.raises(IndicoAuthenticationFailed):
        client = IndicoClient(IndicoConfig(api_token="askldjflkasdjf"))


def test_disable_cookie_domain(indico):
    client = IndicoClient(IndicoConfig(_disable_cookie_domain=True))
    assert client.config._disable_cookie_domain == True
    cookies = client._http.request_session.cookies
    auth_token = next(c for c in cookies if c.name == "auth_token")
    assert auth_token.domain == ""
    response = client.call(
        HTTPRequest(method=HTTPMethod.GET, path="/auth/users/details")
    )
    assert "email" in response


def test_cookie_reset(indico):
    client = IndicoClient(IndicoConfig(_disable_cookie_domain=True))
    client._http.get_short_lived_access_token()
    cookies = client._http.request_session.cookies
    auth_token = next(c for c in cookies if c.name == "auth_token")
    assert auth_token.domain == ""
    response = client.call(
        HTTPRequest(method=HTTPMethod.GET, path="/auth/users/details")
    )
    assert "email" in response


def test_graphql_request(indico):
    client = IndicoClient()
    response = client.call(
        GraphQLRequest(
            query="""
    query {
	    modelGroups{
            modelGroups{
                id
            }
        }
    }
    """
        )
    )
    assert "modelGroups" in response


def test_graphql_with_ids(org_annotate_dataset):
    client = IndicoClient()
    response = client.call(
        GraphQLRequest(
            query="""
            query datasetQueries($id: Int!) {
	            dataset(id: $id){
                    id
                }
            }
        """,
            variables={"id": org_annotate_dataset.id},
        )
    )

    assert "dataset" in response



import json
import time
from pathlib import Path

import pandas as pd
import pytest

from indico.client import IndicoClient
from indico.errors import IndicoError
from indico.queries import AddModelGroupComponent, CreateWorkflow
from indico.queries.datasets import CreateDataset, GetDataset
from indico.queries.questionnaire import (
    AddLabels,
    GetQuestionnaire,
    GetQuestionnaireExamples,
)
from indico.types import ModelTaskType, NewLabelsetArguments, Workflow
from indico.types.questionnaire import Example, Questionnaire


@pytest.fixture
def unlabeled_questionnaire(indico):
    client = IndicoClient()
    datasets_dir = Path(__file__).parents[1] / "data"
    files = [str(datasets_dir / f"pdf{f}.pdf") for f in range(3)]

    dataset = client.call(
        CreateDataset(name=f"CreateDataset-test-{int(time.time())}", files=files)
    )
    workflow: Workflow = client.call(
        CreateWorkflow(
            name=f"CreateWorkflow-test{int(time.time())}", dataset_id=dataset.id
        )
    )
    after_component_id = workflow.component_by_type("INPUT_OCR_EXTRACTION").id
    source_col_id = dataset.datacolumn_by_name("text").id
    classifier_name = f"CreateDatasetTeach-test-{int(time.time())}"

    new_labelset_args = {
        "datacolumn_id": source_col_id,
        "name": classifier_name,
        "num_labelers_required": 1,
        "task_type": ModelTaskType.CLASSIFICATION,
        "target_names": ["A", "B", "C"],
    }

    questionnaire = client.call(
        AddModelGroupComponent(
            name=classifier_name,
            dataset_id=dataset.id,
            workflow_id=workflow.id,
            new_labelset_args=NewLabelsetArguments(**new_labelset_args),
            source_column_id=source_col_id,
            after_component_id=after_component_id,
        )
    )
    questionnaire_id = questionnaire.model_group_by_name(
        classifier_name
    ).model_group.questionnaire_id
    return {
        "dataset": dataset,
        "questionnaire_id": questionnaire_id,
    }


def test_create_questionnaire_no_labels(unlabeled_questionnaire):
    assert isinstance(unlabeled_questionnaire["questionnaire_id"], int)


def test_create_questionnaire_labeled(indico):
    client = IndicoClient()
    datasets_dir = Path(__file__).parents[1] / "data"
    csv_path = datasets_dir / "staffer_large.csv"
    files = [str(datasets_dir / f"pdf{f}.pdf") for f in range(3)]

    dataset = client.call(
        CreateDataset(name=f"CreateDataset-test-{int(time.time())}", files=files)
    )
    workflow: Workflow = client.call(
        CreateWorkflow(
            name=f"CreateWorkflow-test{int(time.time())}", dataset_id=dataset.id
        )
    )
    source_col_id = dataset.datacolumn_by_name("text").id
    after_component_id = workflow.component_by_type("INPUT_OCR_EXTRACTION").id
    csv = pd.read_csv(csv_path)
    data = {
        string: json.loads(label) for string, label in zip(csv["text"], csv["labels"])
    }
    classifier_name = f"CreateDatasetTeach-test-{int(time.time())}"
    targets = list(set(t["label"] for sample in data.values() for t in sample))
    new_labelset_args = {
        "datacolumn_id": source_col_id,
        "name": classifier_name,
        "num_labelers_required": 1,
        "task_type": ModelTaskType.CLASSIFICATION,
        "target_names": targets,
    }

    response = client.call(
        AddModelGroupComponent(
            name=classifier_name,
            dataset_id=dataset.id,
            workflow_id=workflow.id,
            new_labelset_args=NewLabelsetArguments(**new_labelset_args),
            source_column_id=source_col_id,
            after_component_id=after_component_id,
        )
    )

    assert isinstance(response, Workflow)


def test_get_nonexistent_questionnaire(indico):
    client = IndicoClient()
    with pytest.raises(IndicoError):
        client.call(GetQuestionnaire(123454321))


def test_get_questionnaire(indico, unlabeled_questionnaire):
    client = IndicoClient()
    response = client.call(
        GetQuestionnaire(unlabeled_questionnaire["questionnaire_id"])
    )
    assert isinstance(response, Questionnaire)
    assert response.id == unlabeled_questionnaire["questionnaire_id"]
    assert not response.odl
    assert response.num_total_examples == 3
    assert response.num_fully_labeled == 0


def test_get_examples(indico, unlabeled_questionnaire):
    client = IndicoClient()
    examples = client.call(
        GetQuestionnaireExamples(
            questionnaire_id=unlabeled_questionnaire["questionnaire_id"], num_examples=3
        )
    )
    assert len(examples) == 3
    for example in examples:
        assert isinstance(example, Example)
        assert isinstance(example.source, str)
        assert isinstance(example.row_index, int)
        assert isinstance(example.datafile_id, int)


def test_add_labels(indico, unlabeled_questionnaire):
    client = IndicoClient()
    questionnaire = client.call(
        GetQuestionnaire(questionnaire_id=unlabeled_questionnaire["questionnaire_id"])
    )
    targets = questionnaire.question.labelset.target_names
    example = client.call(
        GetQuestionnaireExamples(
            questionnaire_id=unlabeled_questionnaire["questionnaire_id"], num_examples=1
        )
    )
    targets = [
        {
            "clsId": next(t.id for t in targets if t.name == "A"),
            "spans": [{"start": 0, "end": 10, "pageNum": 0}],
        }
    ]
    labels = [
        {
            "exampleId": example[0].id,
            "targets": targets,
        }
    ]
    dataset_id = unlabeled_questionnaire["dataset"].id
    dataset = client.call(GetDataset(id=dataset_id))
    labelset_id = dataset.labelsets[0].id

    client.call(
        AddLabels(
            labelset_id=labelset_id,
            dataset_id=dataset_id,
            labels=labels,
        )
    )
    questionnaire = client.call(
        GetQuestionnaire(unlabeled_questionnaire["questionnaire_id"])
    )
    assert questionnaire.num_total_examples == 3
    assert questionnaire.num_fully_labeled == 1



import datetime
from typing import Any, Iterable, List, Mapping, Union

from indico.errors import IndicoInputError


def or_(*args: Iterable[Mapping[str, Any]]):
    return {"OR": list(args)}


def and_(*args: Iterable[Mapping[str, Any]]):
    return {"AND": list(args)}


class Filter(dict):
    """
    Base filter class that allows users to construct filter statements for
    GraphQL queries. Search keys are constrained by the implementing subclasses
    If multiple arguments are supplied, they are treated as arg1 AND arg2 AND ...
    """

    __options__ = None

    def __init__(self, **kwargs):
        kwargs = {k: v for k, v in kwargs.items() if v is not None}
        if not kwargs:
            raise IndicoInputError(f"One of {self.__options__} must be specified")
        self.update(and_(kwargs) if len(kwargs) > 1 else kwargs)


class DatasetFilter(Filter):
    """
    Create a Filter when querying for Datasets via datasetsPage.

    Args:
        name (str): dataset name by which to filter
    Returns:
        dict containing query filter parameters
    """

    __options__ = ("name",)

    def __init__(self, name: str):
        super().__init__(name=name)


class SubmissionReviewFilter(Filter):
    __options__ = ("rejected", "created_by", "review_type")

    def __init__(
        self,
        rejected: Union[bool, None] = None,
        created_by: Union[int, None] = None,
        review_type: Union[str, None] = None,
    ):
        kwargs = {
            "rejected": rejected,
            "createdBy": created_by,
            "reviewType": review_type.upper() if review_type else review_type,
        }

        super().__init__(**kwargs)


class DateRangeFilter(dict):
    """
    Create a Filter when querying for Submissions within a certain date range
    Args:
        filter_from (str): A valid string representation of a datetime for start date to filter
        filter_to (str): A valid string representation of a datetime for end date to filter
    """

    def __init__(
        self, filter_from: Union[str, None] = None, filter_to: Union[str, None] = None
    ):
        kwargs = {"from": filter_from, "to": filter_to}
        self.update(kwargs)


class SubmissionFilter(Filter):
    """
    Create a Filter when querying for WorkflowSubmissions.

    Args:
        file_type (list): submissions with a file type in this list. Options:
            [CSV, PDF, EXCEL, DOC, DOCX, PPT, PPTX, PNG, JPG, TIFF, TXT, RTF, XLS, XLSX, UNKNOWN, MSG, EML]
        input_filename (str): submissions with input file names containing this string
        status (str): submissions in this status. Options:
            [PROCESSING, PENDING_REVIEW, PENDING_ADMIN_REVIEW, COMPLETE, FAILED]
        retrieved (bool): submissions that have been retrieved (True) or not (False)
        reviews (SubmissionReviewFilter): submissions whose completed reviews match this review filter
        review_in_progress (bool): submissions where a review is in progress (True) or not (False)
        files_deleted (bool): submissions that have had their internal files removed (True) or not (False)
        created_at (DateRangeFilter): submissions created during given time range
        updated_at (DateRangeFilter): submissions updated during given time range
    Returns:
        dict containing query filter parameters
    """

    __options__ = (
        "file_type",
        "input_filename",
        "status",
        "retrieved",
        "reviews",
        "review_in_progress",
        "files_deleted",
        "created_at",
        "updated_at",
    )

    def __init__(
        self,
        file_type: Union[List[str], None] = None,
        input_filename: Union[str, None] = None,
        status: Union[str, None] = None,
        retrieved: Union[bool, None] = None,
        reviews: Union[SubmissionReviewFilter, None] = None,
        review_in_progress: Union[bool, None] = None,
        files_deleted: Union[bool, None] = None,
        created_at: Union[DateRangeFilter, None] = None,
        updated_at: Union[DateRangeFilter, None] = None,
    ):
        kwargs = {
            "filetype": file_type,
            "inputFilename": input_filename,
            "status": status.upper() if status else status,
            "retrieved": retrieved,
            "reviews": reviews,
            "reviewInProgress": review_in_progress,
            "filesDeleted": files_deleted,
            "createdAt": created_at,
            "updatedAt": updated_at,
        }

        super().__init__(**kwargs)


class ModelGroupExampleFilter(Filter):
    """
    Create a Filter when querying for examples associated with model groups.

    Args:
        file_name (str): examples with input file names containing this string
        partial (bool): examples that are or are not partially labeled
        status (str): submissions in this status. Options:
            [COMPLETE, INCOMPLETE]
        text_search (bool): examples that contain this substring in their text
    Returns:
        dict containing query filter parameters
    """

    # TODO: extend to support full filter list
    __options__ = ("file_name", "partial", "status", "text_search")

    def __init__(
        self,
        file_name: Union[str, None] = None,
        partial: Union[bool, None] = None,
        status: Union[str, None] = None,
        text_search: Union[str, None] = None,
    ):
        kwargs = {
            "fileName": file_name,
            "partial": partial,
            "textSearch": text_search,
        }
        kwargs["status"] = status.upper() if status else None

        super().__init__(**kwargs)


class UserMetricsFilter(Filter):
    """
    Create a Filter when querying for UserSnapshots.

    Args:
        user_id (int): username to filter on
        user_email (str): email to filter for
    Returns:
        dict containing query filter parameters
    """

    __options__ = ("user_id", "user_email")

    def __init__(
        self, user_id: Union[int, None] = None, user_email: Union[str, None] = None
    ):
        kwargs = {"userId": user_id, "userEmail": user_email}

        super().__init__(**kwargs)


class DocumentReportFilter(Filter):
    """
    Create a filter for the DocumentReport query.

    Args:
        workflow_id (int): workflow id
        submission_id (int): submission id
        status (str): submission status
        created_at_start_date (datetime): earliest creation date
        created_at_end_date (datetime): latest creation date
        updated_at_start_date (datetime): earliest update date
        updated_at_end_date (datetime): latest update date
    Returns:
        dict containing query filter parameters
    """

    __options__ = (
        "workflow_id",
        "submission_id",
        "status",
        "created_at_start_date",
        "created_at_end_date",
        "updated_at_start_date",
        "updated_at_end_date",
    )

    def __init__(
        self,
        submission_id: Union[int, None] = None,
        workflow_id: Union[int, None] = None,
        status: Union[str, None] = None,
        created_at_start_date: Union[datetime.datetime, None] = None,
        created_at_end_date: Union[datetime.datetime, None] = None,
        updated_at_start_date: Union[datetime.datetime, None] = None,
        updated_at_end_date: Union[datetime.datetime, None] = None,
    ):
        kwargs = {"workflowId": workflow_id, "id": submission_id, "status": status}
        if created_at_end_date and not created_at_start_date:
            raise IndicoInputError("Must specify created_at_start_date")
        if created_at_start_date:
            kwargs["createdAt"] = {
                "from": created_at_start_date.strftime("%Y-%m-%d"),
                "to": (
                    created_at_end_date.strftime("%Y-%m-%d")
                    if created_at_end_date is not None
                    else datetime.datetime.now().strftime("%Y-%m-%d")
                ),
            }

        if updated_at_end_date and not updated_at_start_date:
            raise IndicoInputError("Must specify updated_at_start_date")
        if updated_at_start_date is not None:
            kwargs["updatedAt"] = {
                "from": updated_at_start_date.strftime("%Y-%m-%d"),
                "to": (
                    updated_at_end_date.strftime("%Y-%m-%d")
                    if updated_at_end_date is not None
                    else datetime.datetime.now().strftime("%Y-%m-%d")
                ),
            }
        super().__init__(**kwargs)



import typing as t
from indico.types import BaseType


class TaskBlueprint(BaseType):
    id: int
    name: str
    component_family: str
    icon: str
    description: str
    footer: str
    tags: t.List[str]
    enabled: bool
    config: t.Dict



"""
Setup for indico apis
"""
import versioneer
from pathlib import Path
from setuptools import setup, find_packages

setup(
    name="indico-client",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(exclude=["tests"]),
    description="""A Python Wrapper for indico app API.""",
    license="MIT License (See LICENSE)",
    long_description=open(Path(__file__).parent.absolute() / "README.rst").read(),
    url="https://github.com/IndicoDataSolutions/indico-client-python",
    author="indico",
    author_email="engineering@indico.io",
    tests_require=["pytest>=5.2.1", "requests-mock>=1.8.0", "pytest-asyncio"],
    install_requires=[
        "requests>=2.22.0",
        "setuptools>=41.4.0",
        'importlib-metadata ~= 1.0 ; python_version < "3.8"',
        "deprecation>=2.1.0",
        "jsons",
        "aiohttp[speedups]"
    ],
    extras_require={
        "all": [
            "msgpack>=0.5.6",
            "msgpack-numpy==0.4.4.3",
            "numpy>=1.16.0",
            "pandas>=1.0.3",
        ],
        "datasets": [
            "pandas>=1.0.3",
        ],
        "deserialization": [
            "msgpack>=0.5.6",
            "msgpack-numpy==0.4.4.3",
            "numpy>=1.16.0",
        ],
        "exports": [
            "pandas>=1.0.3",
        ],
    }
)



import pytest
from datetime import datetime

from indico.errors import IndicoInputError
from indico.filters import Filter, SubmissionFilter, and_, or_, DocumentReportFilter


def test_filter():
    with pytest.raises(IndicoInputError):
        Filter()
    f = Filter(key="value")
    assert dict(f) == {"key": "value"}
    f = Filter(key="value", key2="value2")
    assert dict(f) == {"AND": [{"key": "value", "key2": "value2"}]}


@pytest.mark.parametrize("fn,key", [(and_, "AND"), (or_, "OR")])
def test_builder(fn, key):
    f = Filter(key="value")
    f2 = Filter(key2="value2")
    assert fn(f) == {key: [f]}
    assert fn(f, f2) == {key: [f, f2]}


def test_submission_filter():
    f = SubmissionFilter(status="complete")
    assert dict(f) == {"status": "COMPLETE"}


def test_doc_report_filter():
    todays_date = datetime.now().strftime("%Y-%m-%d")

    filter_opts = DocumentReportFilter(created_at_start_date=datetime(2021, 7, 1))
    assert filter_opts['createdAt']['to'] == todays_date
    with pytest.raises(IndicoInputError):
        DocumentReportFilter(created_at_end_date=datetime.now())

    filter_opts = DocumentReportFilter(updated_at_start_date=datetime(2021, 8, 1))
    assert filter_opts['updatedAt']['to'] == todays_date
    with pytest.raises(IndicoInputError):
        DocumentReportFilter(updated_at_end_date=datetime.now())



import logging
import pytest
import os

from indico.client import IndicoClient
from indico.queries.datasets import GetAvailableOcrEngines
from .data.datasets import *
from .data.async_datasets import *

logging.getLogger("indico").setLevel(logging.DEBUG)


def pytest_addoption(parser):
    parser.addoption(
        "--host",
        action="store",
        default="dev-ci.us-east-2.indico-dev.indico.io",
        help="indico ipa host",
    )


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "ocr(ocr_engine): skip test if this OCR engine isn't available on the cluster",
    )


@pytest.fixture(scope="module")
def indico(request):
    host = request.config.getoption("--host")
    os.environ["INDICO_HOST"] = host


@pytest.fixture(autouse=True)
def skip_by_ocr(request):
    client = IndicoClient()
    ocr_engines = [e.name for e in client.call(GetAvailableOcrEngines())]
    if request.node.get_closest_marker("ocr"):
        ocr_engine = request.node.get_closest_marker("ocr").args[0].upper()
        if ocr_engine not in ocr_engines:
            pytest.skip(
                "skipped because cluster does not support OCR engine {}".format(
                    ocr_engine
                )
            )



# -*- coding: utf-8 -*-

import json
import tempfile
from pathlib import Path
from typing import Dict, List, Optional, Union

import jsons

from indico.client.request import (
    Delay,
    GraphQLRequest,
    HTTPMethod,
    HTTPRequest,
    PagedRequest,
    RequestChain,
)
from indico.errors import IndicoInputError, IndicoNotFound
from indico.filters import DatasetFilter
from indico.queries.storage import UploadBatched, UploadImages
from indico.types.dataset import (
    Dataset,
    EmailOptions,
    OcrEngine,
    OcrInputLanguage,
    OmnipageOcrOptionsInput,
    ReadApiOcrOptionsInput,
)


class ListDatasets(PagedRequest):
    """
    List all of your datasets

    Options:
        limit (int, default=100): Max number of datasets to retrieve

    Returns:
        List[Dataset]
    """

    query = """
        query ListDatasets(
            $filters: DatasetFilter,
            $limit: Int,
            $orderBy: DATASET_COLUMN_ENUM,
            $desc: Boolean,
            $after: Int
        ){
            datasetsPage(
                filters: $filters,
                limit: $limit
                orderBy: $orderBy,
                desc: $desc,
                after: $after
            ) {
                datasets {
                    id
                    name
                    rowCount
                }
                pageInfo {
                    hasNextPage
                    endCursor
                }
            }
        }
    """

    def __init__(
        self,
        *,
        filters: Optional[Union[Dict, DatasetFilter]] = None,
        limit: int = 100,
        order_by: str = "ID",
        desc: bool = False,
    ):
        super().__init__(
            self.query,
            variables={
                "filters": filters,
                "limit": limit,
                "orderBy": order_by,
                "desc": desc,
            },
        )

    def process_response(self, response) -> List[Dataset]:
        response = super().process_response(response)
        return [Dataset(**dataset) for dataset in response["datasetsPage"]["datasets"]]


class GetDataset(GraphQLRequest):
    """
    Retrieve a dataset description object

    Args:
        id (int): id of the dataset to query

    Returns:
        Dataset object
    """

    query = """
        query GetDataset($id: Int) {
            dataset(id: $id) {
                id
                name
                rowCount
                status
                permissions
                datacolumns {
                    id
                    name
                }
                labelsets{
                    id
                    name
                }
            }
        }
    """

    def __init__(self, id: int):
        super().__init__(self.query, variables={"id": id})

    def process_response(self, response) -> Dataset:
        response = super().process_response(response)
        if "dataset" not in response or not isinstance(response["dataset"], dict):
            raise IndicoNotFound("Failed to find dataset")
        return Dataset(**response["dataset"])


class GetDatasetFileStatus(GetDataset):
    """
    Get the status of dataset file upload

    Args:
        id (int): id of the dataset to query

    Returns:
        status (str): DOWNLOADED or FAILED
    """

    query = """
        query DatasetUploadStatus($id: Int!) {
            dataset(id: $id) {
                id
                status
                files {
                    id
                    name
                    deleted
                    fileSize
                    rainbowUrl
                    fileType
                    fileHash
                    status
                    statusMeta
                    failureType
                }
            }
        }
    """


class GetDatasetStatus(GraphQLRequest):
    """
    Get the status of a dataset

    Args:
        id (int): id of the dataset to query

    Returns:
        status (str): COMPLETE or FAILED
    """

    query = """
        query datasetStatus($id: Int!) {
            dataset(id: $id) {
                status
            }
        }
    """

    def __init__(self, id: int):
        super().__init__(self.query, variables={"id": id})

    def process_response(self, response) -> str:
        return response["data"]["dataset"]["status"]


class CreateDataset(RequestChain):
    """
    Create a dataset and upload the associated files.

    Args:
        name (str): Name of the dataset.
        files (List[str]): List of path names to the dataset files.
        wait (bool, optional): Wait for the dataset to upload and finish. Defaults to True.
        dataset_type (str, optional): Type of dataset to create [TEXT, DOCUMENT, IMAGE]. Defaults to TEXT.
        from_local_images (bool, optional): Flag whether files are local images or not. Defaults to False.
        image_filename_col (str, optional): Image filename column. Defaults to 'filename'.
        batch_size (int, optional): Size of file batch to upload at a time. Defaults to 20.
        ocr_engine (OcrEngine, optional): Specify an OCR engine [OMNIPAGE, READAPI, READAPI_V2, READAPI_TABLES_V1]. Defaults to None.
        omnipage_ocr_options (OmnipageOcrOptionsInput, optional): If using Omnipage, specify Omnipage OCR options. Defaults to None.
        read_api_ocr_options: (ReadApiOcrOptionsInput, optional): If using ReadAPI, specify ReadAPI OCR options. Defaults to None.
        request_interval (int or float, optional): The maximum time in between retry calls when waiting. Defaults to 5 seconds.

    Returns:
        Dataset object

    """

    previous = None

    def __init__(
        self,
        name: str,
        files: List[str],
        wait: bool = True,
        dataset_type: str = "TEXT",
        from_local_images: bool = False,
        image_filename_col: str = "filename",
        batch_size: int = 20,
        ocr_engine: OcrEngine = None,
        omnipage_ocr_options: OmnipageOcrOptionsInput = None,
        read_api_ocr_options: ReadApiOcrOptionsInput = None,
        request_interval: Union[int, float] = 5,
        email_options: EmailOptions = None,
    ):
        self.files = files
        self.name = name
        self.wait = wait
        self.dataset_type = dataset_type
        self.from_local_images = from_local_images
        self.image_filename_col = image_filename_col
        self.batch_size = batch_size
        self.ocr_engine = ocr_engine
        self.omnipage_ocr_options = omnipage_ocr_options
        self.read_api_ocr_options = read_api_ocr_options
        self.request_interval = request_interval
        self.email_options = email_options
        if omnipage_ocr_options is not None and read_api_ocr_options is not None:
            raise IndicoInputError(
                "Must supply either omnipage or readapi options but not both."
            )
        super().__init__()

    def requests(self):
        if self.from_local_images:
            self.dataset_type = "IMAGE"

            try:
                import pandas as pd
            except ImportError as error:
                raise RuntimeError(
                    "creating image datasets requires additional dependencies:"
                    "`pip install indico-client[datasets]`"
                ) from error

            # Assume image filenames are in the same directory as the csv with
            # image labels and that there is a column representing their name
            df = pd.read_csv(self.files)
            img_filenames = df[self.image_filename_col].tolist()
            img_filepaths = [
                str(Path(self.files).parent / imgfn) for imgfn in img_filenames
            ]
            yield UploadBatched(
                img_filepaths,
                batch_size=self.batch_size,
                request_cls=UploadImages,
            )
            df["urls"] = self.previous
            with tempfile.TemporaryDirectory() as tmpdir:
                image_csv_path = str(Path(tmpdir) / "image_urls.csv")
                df.to_csv(image_csv_path)
                yield _UploadDatasetFiles(files=[image_csv_path])
        else:
            yield UploadBatched(
                files=self.files,
                batch_size=self.batch_size,
                request_cls=_UploadDatasetFiles,
            )
        file_metadata = self.previous
        yield CreateEmptyDataset(
            name=self.name,
            dataset_type=self.dataset_type,
            readapi_ocr_options=self.read_api_ocr_options,
            omnipage_ocr_options=self.omnipage_ocr_options,
            ocr_engine=self.ocr_engine,
            email_options=self.email_options,
        )
        yield _AddFiles(
            dataset_id=self.previous.id, metadata=file_metadata, autoprocess=True
        )
        dataset_id = self.previous.id
        yield GetDatasetFileStatus(id=dataset_id)
        if self.wait is True:
            while not all(
                [f.status in ["PROCESSED", "FAILED"] for f in self.previous.files]
            ):
                yield GetDatasetFileStatus(id=dataset_id)
                yield Delay(seconds=self.request_interval)
        yield GetDataset(id=dataset_id)


class RemoveDatasetFile(GraphQLRequest):
    """
    Remove a file from a dataset by ID.  To retrieve a list of files in a dataset,
    see `GetDatasetFileStatus`.

    Args:
        dataset_id (int): Dataset ID
        file_id (int): Datafile ID (returned by GetDatasetFileStatus)

    Returns:
        Dataset object
    """

    query = """
        mutation RemoveFile($datasetId: Int!, $fileId: Int!){
            deleteDatasetFile(datasetId: $datasetId, fileId: $fileId) {
                id
                status
            }
        }
    """

    def __init__(self, dataset_id: int, file_id: int):
        super().__init__(
            self.query,
            variables={"datasetId": dataset_id, "fileId": file_id},
        )

    def process_response(self, response):
        return Dataset(**super().process_response(response)["deleteDatasetFile"])


class _UploadDatasetFiles(HTTPRequest):
    def __init__(self, files: List[str]):
        super().__init__(
            method=HTTPMethod.POST, path="/storage/files/upload", files=files
        )


class DeleteDataset(GraphQLRequest):
    """
    Delete a dataset

    Args:
        id (int): ID of the dataset

    Returns:
        success (bool): The success of the operation
    """

    query = """
    mutation deleteDataset($id: Int!) {
        deleteDataset(id: $id) {
            success
        }
    }
    """

    def __init__(self, id):
        super().__init__(self.query, variables={"id": id})

    def process_response(self, response):
        return super().process_response(response)["deleteDataset"]["success"]


class CreateEmptyDataset(GraphQLRequest):
    query = """
    mutation($name: String!, $datasetType: DatasetType, $config: DataConfigInput) {
        createDataset(name: $name, datasetType: $datasetType, config: $config ) {
            id
            name
        }
    }
    """

    def __init__(
        self,
        name: str,
        dataset_type: str = None,
        ocr_engine: OcrEngine = None,
        omnipage_ocr_options: OmnipageOcrOptionsInput = None,
        readapi_ocr_options: ReadApiOcrOptionsInput = None,
        email_options: EmailOptions = None,
    ):
        if not dataset_type:
            dataset_type = "TEXT"
        config = None
        if ocr_engine is not None:
            config = {
                "ocrOptions": {
                    "ocrEngine": ocr_engine.name,
                    "omnipageOptions": omnipage_ocr_options,
                    "readapiOptions": readapi_ocr_options,
                },
                "emailOptions": email_options,
            }
        super().__init__(
            self.query,
            variables={
                "name": name,
                "datasetType": dataset_type,
                "config": jsons.dump(
                    config,
                    key_transformer=jsons.KEY_TRANSFORMER_CAMELCASE,
                    strip_nulls=True,
                ),
            },
        )

    def process_response(self, response):
        return Dataset(**super().process_response(response)["createDataset"])


class _AddFiles(GraphQLRequest):
    query = """
    mutation AddFiles($datasetId: Int!, $metadata: JSONString!, $autoprocess: Boolean){
        addDatasetFiles(datasetId: $datasetId, metadataList: $metadata, autoprocess: $autoprocess) {
            id
            status
        }
    }
    """

    def __init__(self, dataset_id: int, metadata: List[str], autoprocess: bool):
        super().__init__(
            self.query,
            variables={
                "datasetId": dataset_id,
                "metadata": json.dumps(metadata),
                "autoprocess": autoprocess,
            },
        )

    def process_response(self, response):
        return Dataset(**super().process_response(response)["addDatasetFiles"])


class AddDatasetFiles(RequestChain):
    """
    Add files to a dataset.

    Args:
        dataset_id (int): ID of the dataset
        files (List[str]): List of pathnames to the dataset files

    Options:
        autoprocess (bool, default=True): Automatically process new dataset files
        wait (bool, default=True): Block while polling for status of files
        batch_size (int, default=20): Batch size for uploading files

    Returns:
        Dataset
    """

    previous = None

    def __init__(
        self,
        dataset_id: int,
        files: List[str],
        autoprocess: bool = True,
        wait: bool = True,
        batch_size: int = 20,
    ):
        self.dataset_id = dataset_id
        self.files = files
        self.wait = wait
        self.batch_size = batch_size
        self.autoprocess = autoprocess
        self.expected_statuses = (
            {"FAILED", "PROCESSED"}
            if autoprocess
            else {"DOWNLOADED", "FAILED", "PROCESSED"}
        )
        super().__init__()

    def requests(self):
        yield UploadBatched(
            files=self.files,
            batch_size=self.batch_size,
            request_cls=_UploadDatasetFiles,
        )
        yield _AddFiles(
            dataset_id=self.dataset_id,
            metadata=self.previous,
            autoprocess=self.autoprocess,
        )
        yield GetDatasetFileStatus(id=self.dataset_id)
        if self.wait:
            while not all(
                f.status in self.expected_statuses for f in self.previous.files
            ):
                yield GetDatasetFileStatus(id=self.previous.id)
                yield Delay()


# Alias for backwards compatibility
AddFiles = AddDatasetFiles


class GetAvailableOcrEngines(GraphQLRequest):
    """
    Fetches and lists the available OCR engines
    """

    query = """query{
        ocrOptions {
            engines{
                name
            }
        }
    }"""

    def __init__(self):
        super().__init__(self.query)

    def process_response(self, response):
        engines = super().process_response(response)["ocrOptions"]["engines"]
        return [OcrEngine[e["name"]] for e in engines]


class GetOcrEngineLanguageCodes(GraphQLRequest):
    """
    Fetches and lists the available languages by name and code for the given OCR Engine

    Args:
        ocr_engine(OcrEngine): The engine to fetch for.
    """

    query = """query{
        ocrOptions {
            engines{
            name
            languages {
                name
                code
                }
            }
        }
    }"""

    def __init__(self, engine: OcrEngine):
        self.engine = engine
        super().__init__(self.query)

    def process_response(self, response):
        data = super().process_response(response)["ocrOptions"]["engines"]
        engine_laguages = next(
            x["languages"] for x in data if x["name"] == self.engine.name
        )
        return [OcrInputLanguage(**option) for option in engine_laguages]



# -*- coding: utf-8 -*-
from typing import Union

from indico.client.request import Delay, GraphQLRequest, RequestChain
from indico.types.jobs import Job
from indico.types.utils import Timer


class _JobStatus(GraphQLRequest):
    query = """
        query JobStatus($id: String) {
            job(id: $id) {
                id
                ready
                status
            }
        }
    """

    def __init__(self, id):
        super().__init__(self.query, variables={"id": id})

    def process_response(self, response):
        return Job(**super().process_response(response)["job"])


class _JobStatusWithResult(GraphQLRequest):
    query = """
        query JobStatus($id: String) {
            job(id: $id) {
                id
                ready
                status
                result
            }
        }
    """

    def __init__(self, id):
        super().__init__(self.query, variables={"id": id})

    def process_response(self, response):
        return Job(**super().process_response(response)["job"])


class JobStatus(RequestChain):
    """
    Status of a Job in the Indico Platform.

    JobStatus is used to either wait for completion or
    query the status of an asynchronous job in the Indico Platform.

    Args:
        id (int): ID of the job to query for status.
        wait (bool, optional): Whether to wait for the job to complete. Defaults to True.
        request_interval (int or float, optional): The maximum time in between retry calls when waiting. Defaults to 0.2.
        timeout (float or int, optional): Timeout after this many seconds.
            Ignored if not `wait`. Defaults to None.

    Returns:
        Job: With the job result available in a result attribute. Note that the result
        will often be JSON but can also be a dict with the URL of a StorageObject on
        the Indico Platform.

    Raises:
        IndicoTimeoutError: If `wait` is True, this error is raised if job has not
            completed after `timeout` seconds
    """

    previous: Job = None

    def __init__(
        self,
        id: str,
        wait: bool = True,
        request_interval: Union[int, float] = 0.2,
        timeout: Union[int, float] = None,
    ):
        self.id = id
        self.wait = wait
        self.request_interval = request_interval
        self.timeout = timeout

    def requests(self):
        yield _JobStatus(id=self.id)
        if self.wait:
            timer = None
            if self.timeout is not None:
                timer = Timer(self.timeout)
            # Check status of job until done if wait == True
            while not (
                (self.previous.status in ["SUCCESS"] and self.previous.ready)
                or self.previous.status
                in [
                    "FAILURE",
                    "REJECTED",
                    "REVOKED",
                    "IGNORED",
                    "RETRY",
                ]
            ):
                if timer:
                    timer.check()
                yield Delay(seconds=self.request_interval)
                yield _JobStatus(id=self.id)
            yield _JobStatusWithResult(id=self.id)



# -*- coding: utf-8 -*-

import json
from typing import List

from indico.client.request import RequestChain, GraphQLRequest, HTTPMethod, HTTPRequest
from indico.types.jobs import Job
from indico.queries.storage import UploadDocument, UploadBatched


class _DocumentExtraction(GraphQLRequest):

    query = """
        mutation($files: [FileInput], $jsonConfig: JSONString, $ocrEngine: OCREngine) {
            documentExtraction(files: $files, jsonConfig: $jsonConfig, ocrEngine: $ocrEngine) {
                jobIds
            }
        }
    """

    def __init__(self, files, json_config={"preset_config": "legacy"}, ocr_engine=None):
        if json_config and type(json_config) == dict:
            json_config = json.dumps(json_config)
        super().__init__(
            query=self.query, variables={"files": files, "jsonConfig": json_config, "ocrEngine": ocr_engine}
        )

    def process_response(self, response):
        jobs = super().process_response(response)["documentExtraction"]["jobIds"]
        if jobs:
            return [Job(id=j) for j in jobs]
        else:
            return []


class DocumentExtraction(RequestChain):
    """
    Extract raw text from PDF or TIF files.

    DocumentExtraction performs Optical Character Recognition (OCR) on PDF or TIF files to
    extract raw text for model training and prediction.

    Args:
        files= (List[str]): Pathnames of one or more files to OCR
        json_config (dict or JSON str): Configuration settings for OCR. See Notes below.
        upload_batch_size (int): Size of batches for document upload if uploading many documents
        ocr_engine (str): Denotes which ocr engine to use. Defaults to OMNIPAGE.

    Returns:
        Job object

    Notes:
        DocumentExtraction is extremely configurable. Four preset configurations are provided:

        simple - Provides a simple and fast response for native PDFs (3-5x faster). Will NOT work with scanned PDFs.

        legacy - Provided to mimic the behavior of Indico's older pdf_extraction function. Use this if your model was trained with data from the older pdf_extraction.

        detailed - Provides detailed bounding box information on tokens and characters. Returns data in a nested format at the document level with all metadata included.

        ondocument - Provides detailed information at the page-level in an unnested format.

        standard - Provides page text and block text/position in a nested format.

        For more information, please reference the Indico knowledgebase article on OCR:
        https://docs.indicodata.ai/articles/documentation-publication/ocr
    """

    def __init__(
        self, files: List[str], json_config: dict = None, upload_batch_size: int = None, ocr_engine: str = "OMNIPAGE"
    ):
        self.files = files
        self.json_config = json_config
        self.upload_batch_size = upload_batch_size
        self.ocr_engine = ocr_engine

    def requests(self):
        if self.upload_batch_size:
            yield UploadBatched(
                files=self.files,
                batch_size=self.upload_batch_size,
                request_cls=UploadDocument,
            )
        else:
            yield UploadDocument(files=self.files)
        yield _DocumentExtraction(files=self.previous, json_config=self.json_config, ocr_engine=self.ocr_engine)



from enum import Enum
from typing import List, Optional

from indico.errors import IndicoInputError
from indico.types.base import BaseType
from indico.types.datafile import Datafile


class DataColumn(BaseType):
    """
    A DataColumn in the Indico Platform

    Attributes:
        id (int): The DataColumn id
        name (str): DataColumn name
    """

    id: int
    name: str


class LabelSet(BaseType):
    """
    A LabelSet in the Indico Platform

    Attributes:
        id (int): The LabelSet id
        name (str): LabelSet name
    """

    id: int
    name: str


class Dataset(BaseType):
    """
    A Dataset in the Indico Platform

    Attributes:
        id (int): The Dataset id
        name (str): Dataset name
        row_count (int): Number of rows in the dataset
        permissions (str): Permissions on the dataset
        files (List[Datafile]): Names of the file(s) included in the dataset
        labelsets (List[LabelSet]): LabelSets associated with this dataset
        datacolumns (List[DataColumn]): DataColumn(s) of the dataset
    """

    id: int
    name: str
    row_count: int
    status: str
    permissions: str
    files: List[Datafile]
    labelsets: List[LabelSet]
    datacolumns: List[DataColumn]

    def labelset_by_name(self, name: str) -> LabelSet:
        if name not in [lab.name for lab in self.labelsets]:
            raise IndicoInputError(
                f"No labelset found for {name}. Current labelset names include {[lab.name for lab in self.labelsets]}."
            )
        return next(lab for lab in self.labelsets if lab.name == name)

    def datacolumn_by_name(self, name: str) -> DataColumn:
        if name not in [datacol.name for datacol in self.datacolumns]:
            raise IndicoInputError(
                f"No datacolumn found for {name}. Current datacolumn names include {[datacol.name for datacol in self.datacolumns]}."
            )
        return next(datacol for datacol in self.datacolumns if datacol.name == name)


class TableReadOrder(Enum):
    ROW = 0
    COLUMN = 1


class OcrEngine(Enum):
    """
    Enum representing available OCR engines.
    """

    OMNIPAGE = 0
    READAPI = 1
    READAPI_V2 = 2
    READAPI_TABLES_V1 = 3


class OmnipageOcrOptionsInput(BaseType):
    """
    Omnipage specific OCR options for dataset creation.

    Args:
        auto_rotate(bool): auto rotate.
        single_colum(bool): Read table as a single column.
        upscale_images(bool): Scale up low-resolution images.
        languages(List[OmnipageLanguageCode]): List of languages to use in ocr.
        cells(bool): Return table information for post-processing rules
        force_render(bool): Force rednering.
        native_layout(bool): Native layout.
        native_pdf(bool): Native pdf.
        table_read_order(TableReadOrder): Read table by row or column.

    """

    auto_rotate: bool
    single_column: bool
    upscale_images: bool
    languages: List[str]
    cells: bool
    force_render: bool
    native_layout: bool
    native_pdf: bool
    table_read_order: TableReadOrder


class ReadApiOcrOptionsInput(BaseType):
    """
    Read API OCR options.

    Args:
        auto_rotate(bool): Auto rotate
        single_column(bool): Read table as a single column.
        upscale_images(bool): Scale up low resolution images.
        languages(List[str]): List of languages to use.
    """

    auto_rotate: bool
    single_column: bool
    upscale_images: bool
    languages: List[str]


class OcrInputLanguage(BaseType):
    name: str
    code: str


class IncludeSections(BaseType):
    header: Optional[bool]
    body: Optional[bool]
    attachments: Optional[bool]


class EmailOptions(BaseType):
    """
    Email options

    Args:
        include_sections: Sections of the email to include after parsing (header, body, attachments)
        unpack: Unpack an email and treat it as a multi-file Submission
        preserve_body_whitespace: Preserve whitespace in the body of the email
    """

    include_sections: Optional[IncludeSections]
    unpack: Optional[bool]
    preserve_body_whitespace: Optional[bool]


class OcrOptionsInput:
    """
    Input options for OCR engine.
    """

    ocr_engine: OcrEngine
    omnipage_options: OmnipageOcrOptionsInput
    readapi_options: ReadApiOcrOptionsInput



import pytest
from indico.client import IndicoClient
from indico.errors import IndicoInputError
from indico.queries.custom_blueprint import RegisterCustomBlueprint
from indico.types.workflow import ComponentFamily


def test_register_blueprint(indico):
    client = IndicoClient()
    bpreq = RegisterCustomBlueprint(
        component_family=ComponentFamily.OUTPUT, 
        name="Meowtput", 
        description="adds 'cat' to the result file", 
        config={
            "inputs": [{"name": "result_file", "ioClass": "PartitionStream"}],
            "outputs": [{"name": "result_file", "ioClass": "PartitionStream"}],
            "submissionLauncher": {"service": "customizer_default", "name": "cat_output"}
        },
        tags= ["custom"],
        all_access=True,
        footer="version 2"
    )
    bp = client.call(bpreq)
    print(bp.__dict__)
    assert bp.id

def test_register_blueprint_bad_config(indico):
    with pytest.raises(IndicoInputError):
        RegisterCustomBlueprint(
            component_family=ComponentFamily.OUTPUT, 
            name="Meowtput", 
            description="adds 'cat' to the result file", 
            config={
                "inputs": [{"name": "result_file", "ioClass": "PartitionStream"}],
                "outputs": [{"name": "result_file", "ioClass": "PartitionStream"}],
                "submissionLauncher": {}
            },
            tags= ["custom"],
            all_access=True,
            footer="version 2"
        )



import pytest_asyncio
import time
from pathlib import Path
from indico.client import AsyncIndicoClient
from indico.queries import (
    CreateDataset,
)


@pytest_asyncio.fixture(scope="module")
async def aio_org_annotate_dataset(indico):
    dataset_filepath = str(Path(__file__).parents[0]) + "/org-annotate-labeled.csv"
    async with AsyncIndicoClient() as client:
        response = await client.call(
            CreateDataset(
                name=f"OrgAnnotate-test-{int(time.time())}", files=[dataset_filepath]
            )
        )
    assert response.status == "COMPLETE"
    return response



from indico.types import BaseType


class SubmissionFile(BaseType):
    """
    A Submission File in the Indico Platform.

    Submissions files represent a single document, and can be grouped together under
    a single submission to a submission to a workflow.

    Attributes:
        id (int): The Submission file id
        submission_id (int): The parent Submission id
        filename (str): Name of the original file
        filepath (str): URL of the input datafile within the Indico Platform.
        filetype (str): The file type of the original file; most likely, this is "PDF".
        file_size (int): Size of file, in bytes
        num_pages (int): Number of pages in file
    """

    id: int
    submission_id: int
    filename: str
    filepath: str
    filetype: str
    file_size: int
    num_pages: int
